@@ -1,6 +1,7 @@
 import datetime
 import socket
 import json
+
 from pg_tasks_queue.Config import cfg as config
 
 from sqlalchemy import create_engine
@@ -15,7 +16,7 @@ from sqlalchemy import and_
 from sqlalchemy import or_
 # from sqlalchemy.ext.declarative import declared_attr
 # from sqlalchemy import MetaData
-
+from pg_tasks_queue.Logging import init_logger
 
 import logging
 logger = logging.getLogger(__name__)
@@ -139,7 +140,7 @@ class Config(Base):
 
     __tablename__ = 'config'
 
-    config_id = Column(Integer, primary_key=True)
+    config_id = Column(VARCHAR(20), primary_key=True)
     config = Column(JSONB)
 
     def __repr__(self):
@@ -178,8 +179,8 @@ def get_connection_string():
     return connection_string
 
 
-def init(cfg=None, use_project=True, create=True):
-
+def init(cfg=None, use_project=True, create=True, test=True):
+    init_logger()
     if cfg is None:
         if not isinstance(config.cfg, dict):
             logger.error(f"Error in Database.init(): not isinstance(config.cfg, dict)...")
@@ -240,6 +241,12 @@ def init(cfg=None, use_project=True, create=True):
     if create:
         Base.metadata.create_all(db_engine)
 
+    if test:
+        if not get_session():
+            logger.error(f'Error in Database.init(): not database.get_session()...')
+            return False
+        close_session()
+
     return True
 
 
@@ -291,14 +298,14 @@ def get_html_table(query_list, columns=None, table=None):
     return {'columns': columns, 'values': values, 'types': types}
 
 
-def get_session(autocommit=False):
+def get_session(autocommit=False, expire_on_commit=False):
     global session
     if session is None:
         if db_engine is None:
             logger.error(f'Error in Database.get_session(): db_engine is None...')
             return False
         try:
-            Session = sessionmaker(bind=db_engine)
+            Session = sessionmaker(bind=db_engine, expire_on_commit=expire_on_commit)
             session = Session()
         except Exception as e:
             logger.exception(f'Exception in Database.get_session(): {type(e)}: {str(e)}...')
@@ -321,6 +328,28 @@ def close_session():
         session = None
 
 
+def commit():
+    if not get_session():
+        logger.error(f'Error in Database.commit(): not get_session()...')
+        return
+    session.commit()
+    close_session()
+
+
+def add(_add_value):
+    if not get_session():
+        logger.error(f'Error in Database.commit(): not get_session()...')
+        return
+    session.add(_add_value)
+    session.commit()
+    close_session()
+    return _add_value
+
+
+def update(_update_value):
+    return add(_update_value)
+
+
 def get_table(_value, _class):
     if isinstance(_value, dict):
         _value = _class(**_value)
@@ -341,10 +370,6 @@ def add_common_columns(_value):
 
 
 def add_new_record(_add_value, _class, func=None):
-    if session is None:
-        error = f'Error in Database.add_new_record(): session is None...'
-        logger.error(error)
-        return error
     _add_value = get_table(_add_value, _class)
     if not isinstance(_add_value, _class):
         error = f'Error in Database.add_new_record(): not isinstance(_add_value, {_class.__class__.__name__})...'
@@ -353,13 +378,11 @@ def add_new_record(_add_value, _class, func=None):
     if func is not None:
         _add_value.module = func.__module__
         _add_value.func = func.__name__
-
-    session.add(add_common_columns(_add_value))
-    session.commit()
-    return _add_value
+    _add_value = add_common_columns(_add_value)
+    return add(_add_value)
 
 
-def add_cornjob_task(_add_value, func=None, commit=True):
+def add_cornjob_task(_add_value, func=None):
     return add_new_record(_add_value, CronjobTask, func=func)
 
 
@@ -379,9 +402,22 @@ def get_new_worker():
     return add_new_record(new_worker, Worker)
 
 
+def get_config(config_id):
+    if not get_session():
+        error = f'Error in Database.get_config(): not get_session()...'
+        logger.error(error)
+        return error
+    db_cfg = session.query(Config).filter(Config.config_id == config_id).first()
+
+    session.commit()
+    close_session()
+
+    return db_cfg
+
+
 def get_new_task(worker=None):
-    if session is None:
-        error = f'Error in Database.get_new_task(): session is None...'
+    if not get_session():
+        error = f'Error in Database.get_new_task(): not get_session()...'
         logger.error(error)
         return error
 
@@ -424,12 +460,55 @@ def get_new_task(worker=None):
         task.status = 'working'
 
     session.commit()
+    close_session()
+
+    return task
+
+
+def get_cronjob_tasks():
+    if not get_session():
+        error = f'Error in Database.get_cronjob_tasks(): not get_session()...'
+        logger.error(error)
+        return error
+    if project:
+        filter_value = and_(CronjobTask.project == project, CronjobTask.enabled.is_(True))
+    else:
+        filter_value = CronjobTask.enabled.is_(True)
+
+    cronjob_tasks = session.query(CronjobTask).filter(filter_value).all()
+
+    session.commit()
+    close_session()
+
+    return cronjob_tasks
+
+
+def get_task_for_cronjob(cronjob_task, start_of_min):
+    if not get_session():
+        error = f'Error in Database.get_cronjob_tasks(): not get_session()...'
+        logger.error(error)
+        return error
+
+    if project:
+        filter_value = and_(Task.project == project,
+                            Task.cronjob_task_id == cronjob_task.task_id,
+                            Task.scheduled_time > start_of_min)
+
+    else:
+        filter_value = and_(Task.cronjob_task_id == cronjob_task.task_id,
+                            Task.scheduled_time > start_of_min)
+
+    task = session.query(Task).filter(filter_value).first()
+
+    session.commit()
+    close_session()
+
     return task
 
 
 def get_new_tasks(task_limit, worker=None):
-    if session is None:
-        error = f'Error in Database.get_new_task(): session is None...'
+    if not get_session():
+        error = f'Error in Database.get_new_tasks(): not get_session()...'
         logger.error(error)
         return error
 
@@ -467,12 +546,14 @@ def get_new_tasks(task_limit, worker=None):
         task.status = 'working'
 
     session.commit()
+    close_session()
+
     return tasks
 
 
 def update_worker_task(_result, task, worker):
-    if session is None:
-        error = f'Error in Database.update_worker_task(): session is None...'
+    if not get_session():
+        error = f'Error in Database.update_worker_task(): not get_session()...'
         logger.error(error)
         return error
 
@@ -517,16 +598,20 @@ def update_worker_task(_result, task, worker):
 
     if worker:
         worker.current_task_id = None
+        session.add(worker)
 
     task.finished_time = finished_time
+    session.add(task)
+
     session.commit()
+    close_session()
 
     return None
 
 
 def set_worker_error(error, worker, task):
-    if session is None:
-        error = f'Error in Database.set_worker_error(): session is None...'
+    if not get_session():
+        error = f'Error in Database.set_worker_error(): not get_session()...'
         logger.error(error)
         return error
 
@@ -547,12 +632,16 @@ def set_worker_error(error, worker, task):
         failed_tasks = json.loads(worker.failed_tasks)
         failed_tasks.append(task.task_id)
         worker.failed_tasks = json.dumps(failed_tasks)
+        session.add(task)
 
     worker.status = 'error'
     worker.error = error
     worker.finished_time = finished_time
 
+    session.add(worker)
+
     session.commit()
+    close_session()
 
 # ======================================================================================================================
 
@@ -562,7 +651,6 @@ if __name__ == '__main__':
     ver = '1.2'
     release_date = '2020-06-05 10:00'
     version = f'{name}. ver: {ver} ({release_date})'
-    from pg_tasks_queue.Logging import init_logger
     init_logger()
     logger.warning(version)
 
@@ -571,32 +659,25 @@ if __name__ == '__main__':
     if not init(cfg):
         logger.warning(f"not init()")
     else:
-        if not get_session():
-            logger.warning(f"not get_session()")
-        else:
-            # get_new_task()
+        new_cornjob_task = {'schedule': '*/1 * * * *',
+                            'module': 'Modules.TestTasks',
+                            'func': 'raise_exception',  # raise_exception, sleep_false, sleep
+                            'params': {"counter": 4},
+                            'project': 'project_2',
+                            'max_retry_count': 2,
+                            'enabled': True}
+        # new_cornjob_task = CronjobTask(**new_cornjob_task)
+        # from Modules import TestTasks
+        # new_cornjob_task = add_cornjob_task(new_cornjob_task, TestTasks.raise_exception)
+        new_cornjob_task = add_cornjob_task(new_cornjob_task)
+        print(f'new_cornjob_task: {new_cornjob_task}')
 
-            new_cornjob_task = {'schedule': '*/1 * * * *',
-                                'module': 'Modules.TestTasks',
-                                'func': 'raise_exception',  # raise_exception, sleep_false, sleep
-                                'params': {"counter": 4},
-                                'project': 'project_2',
-                                'max_retry_count': 2,
-                                'enabled': True}
-            # new_cornjob_task = CronjobTask(**new_cornjob_task)
-            # from Modules import TestTasks
-            # new_cornjob_task = add_cornjob_task(new_cornjob_task, TestTasks.raise_exception)
-            new_cornjob_task = add_cornjob_task(new_cornjob_task)
-            print(f'new_cornjob_task: {new_cornjob_task}')
-
-            import datetime
-            new_task = {'module': 'Modules.TestTasks',
-                        'func': 'raise_exception',
-                        'params': {"counter": 4},
-                        'max_retry_count': 20,
-                        'scheduled_time': datetime.datetime.now(),
-                        'priority': 1}
-            new_task = add_task(new_task)
-            print(f'new_task: {new_task}')
-
-            close_session()
+        import datetime
+        new_task = {'module': 'Modules.TestTasks',
+                    'func': 'raise_exception',
+                    'params': {"counter": 4},
+                    'max_retry_count': 20,
+                    'scheduled_time': datetime.datetime.now(),
+                    'priority': 1}
+        new_task = add_task(new_task)
+        print(f'new_task: {new_task}')
