@@ -8,6 +8,7 @@ import queue
 import datetime
 import multiprocessing
 import traceback
+from pg_tasks_queue.Logging import init_logger
 from pg_tasks_queue.Config import cfg as config
 import pg_tasks_queue.Database as database
 from pg_tasks_queue.Kthread import KThread as kthread
@@ -92,96 +93,96 @@ class Worker:
         finally:
             logger.error(error)
 
-    def _do_task(self, process_type):
-        func_name = f'{self.__class__.__name__}.{sys._getframe().f_code.co_name}()'
-        # print(f'{func_name}; os.getpid(): {os.getpid()}; os.getppid(): {os.getppid()}')
-        if not self._started:
-            logger.warning(f'{func_name}; self._started = False; return')
-            return
-
-        exception = None
-        error = None
-        timer = None
-        try:
-            self._current_task = database.get_new_task(self._worker)
-            if isinstance(self._current_task, str):
-                logger.error(f'Error in {func_name}; {self._current_task}')
-                return
-            elif self._current_task is None:
-                return
-
-            # print(f'{func_name}: now={datetime.datetime.now()}; task: {self._current_task}')
-            task_dict = database.get_record_dict(self._current_task)
-            if process_type is None:
-                # timer = threading.Timer(self._timeout_sec, self.stop_woker, args=(func_name,))
-                # timer.setDaemon(False)
-                # timer.start()
-                result_dict = self.worker(task_dict)
-                # timer.cancel()
-                database.update_worker_task(result_dict, self._current_task, self._worker)
-            elif process_type in ['fork', 'thread', 'kthread']:
-                if process_type == 'fork':
-                    worker_queue = multiprocessing.Queue()
-                    worker = multiprocessing.Process(target=self.worker_process, args=(worker_queue, task_dict,))
-                else:
-                    worker_queue = queue.Queue()
-                    if process_type == 'thread':
-                        worker = threading.Thread(target=self.worker_process, args=(worker_queue, task_dict,))
-                        worker.setDaemon(True)
-                    else:
-                        worker = kthread(target=self.worker_process, args=(worker_queue, task_dict,))
-                worker.start()
-                worker.join(timeout=self._timeout_sec)
-                if worker.is_alive():
-                    if process_type in ['fork', 'kthread']:
-                        worker.terminate()
-                        error = f'Error in {func_name}: worker({process_type}).is_alive() => worker.terminate()'
-                        logger.error(error)
-                        database.update_worker_task({'status': 'error', 'result': error}, self._current_task,
-                                                    self._worker)
-                    else:
-                        error = f'Error in {func_name}: worker({process_type}).is_alive() => sys.exit(1)'
-                        logger.error(error)
-                        if self._worker:
-                            database.set_worker_error(error, self._worker, self._current_task)
-                        sys.exit(1)
-                else:
-                    if not worker_queue.empty():
-                        database.update_worker_task(worker_queue.get_nowait(), self._current_task, self._worker)
-                    else:
-                        error = f'Error in {func_name}: queue is empty()...'
-                        logger.error(error)
-                        database.update_worker_task({'status': 'error', 'result': error}, self._current_task,
-                                                    self._worker)
-
-        except Exception as e:
-            exception = e
-            error = f'Exception in {func_name}: {type(e)}: {str(e)}; traceback: {traceback.format_exc()}'
-        finally:
-            if timer:
-                timer.cancel()
-            if exception:
-                if self._raise_exception:
-                    if self._worker:
-                        database.set_worker_error(error, self._worker, self._current_task)
-                    raise exception
-                else:
-                    database.update_worker_task({'status': 'error', 'result': error}, self._current_task,
-                                                self._worker)
-                    logger.error(error)
-            if not self._blocking:
-                if self._life_timeout_sec is not None:
-                    time_delta = datetime.datetime.now() - self._started_timestamp
-                    time_delta_sec = time_delta.total_seconds()
-                    if time_delta_sec > self._life_timeout_sec:
-                        logger.info(f'{func_name}; time_delta_sec({time_delta_sec})'
-                                    f' > life_timeout_sec({self._life_timeout_sec}) => return...')
-                        if self._worker:
-                            self._worker.status = 'finished'
-                            self._worker.finished_time = datetime.datetime.now()
-                            database.session.commit()
-                        return
-                threading.Timer(self._sleep_sec, self._do_task, args=(process_type,)).start()
+    # def _do_task(self, process_type):
+    #     func_name = f'{self.__class__.__name__}.{sys._getframe().f_code.co_name}()'
+    #     # print(f'{func_name}; os.getpid(): {os.getpid()}; os.getppid(): {os.getppid()}')
+    #     if not self._started:
+    #         logger.warning(f'{func_name}; self._started = False; return')
+    #         return
+    #
+    #     exception = None
+    #     error = None
+    #     timer = None
+    #     try:
+    #         self._current_task = database.get_new_task(self._worker)
+    #         if isinstance(self._current_task, str):
+    #             logger.error(f'Error in {func_name}; {self._current_task}')
+    #             return
+    #         elif self._current_task is None:
+    #             return
+    #
+    #         # print(f'{func_name}: now={datetime.datetime.now()}; task: {self._current_task}')
+    #         task_dict = database.get_record_dict(self._current_task)
+    #         if process_type is None:
+    #             # timer = threading.Timer(self._timeout_sec, self.stop_woker, args=(func_name,))
+    #             # timer.setDaemon(False)
+    #             # timer.start()
+    #             result_dict = self.worker(task_dict)
+    #             # timer.cancel()
+    #             database.update_worker_task(result_dict, self._current_task, self._worker)
+    #         elif process_type in ['fork', 'thread', 'kthread']:
+    #             if process_type == 'fork':
+    #                 worker_queue = multiprocessing.Queue()
+    #                 worker = multiprocessing.Process(target=self.worker_process, args=(worker_queue, task_dict,))
+    #             else:
+    #                 worker_queue = queue.Queue()
+    #                 if process_type == 'thread':
+    #                     worker = threading.Thread(target=self.worker_process, args=(worker_queue, task_dict,))
+    #                     worker.setDaemon(True)
+    #                 else:
+    #                     worker = kthread(target=self.worker_process, args=(worker_queue, task_dict,))
+    #             worker.start()
+    #             worker.join(timeout=self._timeout_sec)
+    #             if worker.is_alive():
+    #                 if process_type in ['fork', 'kthread']:
+    #                     worker.terminate()
+    #                     error = f'Error in {func_name}: worker({process_type}).is_alive() => worker.terminate()'
+    #                     logger.error(error)
+    #                     database.update_worker_task({'status': 'error', 'result': error}, self._current_task,
+    #                                                 self._worker)
+    #                 else:
+    #                     error = f'Error in {func_name}: worker({process_type}).is_alive() => sys.exit(1)'
+    #                     logger.error(error)
+    #                     if self._worker:
+    #                         database.set_worker_error(error, self._worker, self._current_task)
+    #                     sys.exit(1)
+    #             else:
+    #                 if not worker_queue.empty():
+    #                     database.update_worker_task(worker_queue.get_nowait(), self._current_task, self._worker)
+    #                 else:
+    #                     error = f'Error in {func_name}: queue is empty()...'
+    #                     logger.error(error)
+    #                     database.update_worker_task({'status': 'error', 'result': error}, self._current_task,
+    #                                                 self._worker)
+    #
+    #     except Exception as e:
+    #         exception = e
+    #         error = f'Exception in {func_name}: {type(e)}: {str(e)}; traceback: {traceback.format_exc()}'
+    #     finally:
+    #         if timer:
+    #             timer.cancel()
+    #         if exception:
+    #             if self._raise_exception:
+    #                 if self._worker:
+    #                     database.set_worker_error(error, self._worker, self._current_task)
+    #                 raise exception
+    #             else:
+    #                 database.update_worker_task({'status': 'error', 'result': error}, self._current_task,
+    #                                             self._worker)
+    #                 logger.error(error)
+    #         if not self._blocking:
+    #             if self._life_timeout_sec is not None:
+    #                 time_delta = datetime.datetime.now() - self._started_timestamp
+    #                 time_delta_sec = time_delta.total_seconds()
+    #                 if time_delta_sec > self._life_timeout_sec:
+    #                     logger.info(f'{func_name}; time_delta_sec({time_delta_sec})'
+    #                                 f' > life_timeout_sec({self._life_timeout_sec}) => return...')
+    #                     if self._worker:
+    #                         self._worker.status = 'finished'
+    #                         self._worker.finished_time = datetime.datetime.now()
+    #                         database.session.commit()
+    #                     return
+    #             threading.Timer(self._sleep_sec, self._do_task, args=(process_type,)).start()
 
     def _do_current_task(self, process_type):
         func_name = f'{self.__class__.__name__}.{sys._getframe().f_code.co_name}()'
@@ -248,7 +249,6 @@ class Worker:
                 if self._raise_exception:
                     if self._worker:
                         database.set_worker_error(error, self._worker, self._current_task)
-                    database.close_session()
                     raise exception
                 else:
                     database.update_worker_task({'status': 'error', 'result': error}, self._current_task,
@@ -261,10 +261,6 @@ class Worker:
             logger.warning(f'{func_name}; self._started = False; return')
             return
 
-        if not database.get_session():
-            logger.error(f'Error in {func_name}: not database.get_session()...')
-            return
-
         new_tasks = database.get_new_tasks(task_limit, self._worker)
         if isinstance(self._current_task, str):
             logger.error(f'Error in {func_name}; {new_tasks}')
@@ -272,17 +268,16 @@ class Worker:
 
         for task in new_tasks:
             task.started_time = datetime.datetime.now()
+            database.update(task)
             if self._worker:
                 self._worker.current_task_id = task.task_id
-            database.session.commit()
+                database.update(self._worker)
             self._current_task = task
             self._do_current_task(process_type)
 
         if self._worker:
             self._worker.current_task_id = None
-
-        database.session.commit()
-        database.close_session()
+            database.update(self._worker)
 
         if not self._blocking:
             if self._life_timeout_sec is not None:
@@ -294,11 +289,12 @@ class Worker:
                     if self._worker:
                         self._worker.status = 'finished'
                         self._worker.finished_time = datetime.datetime.now()
-                        database.session.commit()
+                        database.update(self._worker)
                     return
             threading.Timer(self._sleep_sec, self._do_task, args=(process_type,)).start()
 
     def start(self, cfg=None, config_id=None):
+        init_logger()
         func_name = f'{self.__class__.__name__}.{sys._getframe().f_code.co_name}()'
 
         if cfg is None:
@@ -311,11 +307,6 @@ class Worker:
             logger.error(f'Error in {func_name}: not database.init()...')
             return
 
-        if not database.get_session():
-            error = f'Error in {func_name}: not database.get_session()...'
-            logger.error(error)
-            return error
-
         process_type = None
         task_limit = 20
         loops_limit = 1
@@ -337,8 +328,7 @@ class Worker:
             worker_cfg['task_limit'] = int(worker_cfg.get('task_limit'))
 
         if config_id is not None:
-            db_cfg = database.session.query(database.Config).filter(
-                database.Config.config_id == config_id).first()
+            db_cfg = database.get_config(config_id)
             if db_cfg is None:
                 db_cfg = database.Config(config_id=config_id, config=worker_cfg)
                 database.add_new_record(db_cfg, database.Config)
@@ -353,7 +343,6 @@ class Worker:
         if 'enabled' in worker_cfg:
             if worker_cfg.get('enabled', True) is False:
                 logger.warning(f'Error in {func_name}: enabled if False => return...')
-                database.close_session()
                 return
 
         if 'blocking' in worker_cfg:
@@ -387,8 +376,6 @@ class Worker:
         if create_db_worker:
             self._worker = database.get_new_worker()
 
-        database.close_session()
-
         self._started = True
         self._started_timestamp = datetime.datetime.now()
         if self._blocking:
@@ -424,13 +411,11 @@ class Worker:
 if __name__ == '__main__':
     name = 'Worker'
     ver = '1.3'
-    release_date = '2020-06-08 06:00'
+    release_date = '2020-06-08 13:30'
     version = f'{name}. ver: {ver} ({release_date})'
-    from pg_tasks_queue.Logging import init_logger
     init_logger()
-
     logger.warning(f'Start {version}')
-    Worker().start(config.cfg, config_id=1)
+    Worker().start(config.cfg, config_id='first')
     logger.warning(f'End {version}')
 
 
